"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.staleIfError = exports.staleWhileRevalidate = void 0;
var function_1 = require("fp-ts/function");
var remote_data_ts_1 = require("@devexperts/remote-data-ts");
/**
 * Given a {@link RemoteDataRevalidate}, the stale-while-revalidate refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - keep returning a success value until it is replaced by another success or failure value
 * - keep returning a failure value only as long as it is not being refreshed, in which case
 *   it will fall back to a pending value
 *
 * @see {@link staleIfError}
 */
function staleWhileRevalidate(_a) {
    var current = _a[0], next = _a[1];
    return function_1.pipe(next, remote_data_ts_1.fold(function () { return [current, next]; }, function () { return [remote_data_ts_1.isInitial(current) || remote_data_ts_1.isFailure(current) ? next : current, next]; }, function (e) { return [remote_data_ts_1.failure(e), remote_data_ts_1.initial]; }, function (x) { return [remote_data_ts_1.success(x), remote_data_ts_1.initial]; }));
}
exports.staleWhileRevalidate = staleWhileRevalidate;
/**
 * Given a {@link RemoteDataRevalidate}, the stale-if-error refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - return a failure only if no data has been previously fetched; newer failures replace older ones
 * - keep returning the cached success value indefinitely unless it is replaced by another success value
 *
 * @see {@link staleWhileRevalidate}
 */
function staleIfError(_a) {
    var current = _a[0], next = _a[1];
    return function_1.pipe(next, remote_data_ts_1.fold(function () { return [current, next]; }, function () { return [remote_data_ts_1.isInitial(current) ? next : current, next]; }, function (e) { return [remote_data_ts_1.isSuccess(current) ? current : remote_data_ts_1.failure(e), remote_data_ts_1.initial]; }, function (x) { return [remote_data_ts_1.success(x), remote_data_ts_1.initial]; }));
}
exports.staleIfError = staleIfError;
